/*
*
* Copyright (c) 2004 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Patterns and Strategies for Building Document-Based Web Services"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/

import com.examples.base64.Base64;
import com.examples.base64.clientbindings.*;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;


public class WSDLClient {

    public static void main(String[] args) throws Exception {
        String url = "http://localhost:9090/base64/jaxrpc";
        if (args.length == 1) {
            url = args[0];
        }
        XMLStringService_Impl serviceproxy = new XMLStringService_Impl();
        IStringService_Stub stub = (IStringService_Stub) (serviceproxy.getIStringServicePort());
        stub._setProperty(javax.xml.rpc.Stub.ENDPOINT_ADDRESS_PROPERTY, url);
        sendStringFromFile("purchaseorder.xml", stub);
        sendRawBytesFromFile("purchaseorder.xml", stub);
    }


    /**
     * Sends the contents to the service after reading from a file and base64 encoding the data
     *
     * @param filename
     * @param stub
     * @throws Exception
     */
    public static void sendStringFromFile(String filename, IStringService_Stub stub) throws Exception {
        System.out.println("========================================== sending base64encoded data ========================");
        String filedata = readStringFromFileRaw(filename);
        String base64encoded = encode(filedata);
        SayXMLHello request = new SayXMLHello(base64encoded);
        SayXMLHelloResponse response = stub.sayXMLHello(request);
        String data = response.getResult();
        System.out.println("Reply from server before decoding is  : \n " + data);
        byte[] raw = Base64.decode(data);
        String answer = new String(raw);
        System.out.println("\nAfter decoding , servers reply is  : \n " + answer);
        System.out.println("==================================================================");
    }


    public static void sendRawBytesFromFile(String filename, IStringService_Stub stub) throws Exception {
        System.out.println("========================================== sending raw byte data ========================");
        SayRawHello request = new SayRawHello(readStringFromFileRaw(filename).getBytes());
        SayRawHelloResponse response = stub.sayRawHello(request);
        byte[] raw = response.getResult();
        String answer = new String(raw);
        System.out.println("Server replied back with following raw bytes : \n ");
        for (int i = 0; i < raw.length; i++)
            System.out.print(raw[i]);
        System.out.println("\nRaw bytes from server in string format are : \n " + answer);
        System.out.println("==================================================================");

    }

    /**
     * Reads the contents form a file and returns a String
     *
     * @param filename
     * @return
     * @throws Exception
     */
    private static String readStringFromFileRaw(String filename) throws Exception {
        StringBuffer sb = new StringBuffer();
        BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(filename)));
        String input = br.readLine();
        while (input != null) {
            sb.append(input);
            input = br.readLine();
        }
        br.close();
        String contents = sb.toString();
        return contents;
    }

    private static String encode(String contents) {

        String data = Base64.encode(contents.getBytes());
        return data;
    }
}
